@@ -8,14 +8,20 @@ import sys
 
 from beet import ProjectConfig
 from stouputils.decorators import LogLevels, handle_error
-from stouputils.print import info
+from stouputils.print import GREEN, RED, info, show_version
 
 from .utils import get_project_config
 
 
 @handle_error(message="Error while running 'stewbeet'")
 def main():
-    second_arg: str = sys.argv[1] if len(sys.argv) == 2 else "build"
+    second_arg: str = sys.argv[1].lower() if len(sys.argv) == 2 else ""
+    if second_arg == "" and len(sys.argv) == 1:
+        sys.argv.append("build")
+
+    # Print the version of stewbeet, beet, bolt, mecha, and stouputils
+    if second_arg in ("--version", "-v", "version"):
+        return show_version("stewbeet", primary_color=RED, secondary_color=GREEN)
 
     # Try to find and load the beet configuration file
     cfg: ProjectConfig = get_project_config()
@@ -49,6 +55,11 @@ def main():
             os.remove(definitions_debug)
         info("Cleaning done!")
 
+        # Replace "rebuild" by "build" to continue the process
+        if second_arg == "rebuild":
+            sys.argv[1] = "build"
+
+    # Handle all other commands except "clean"
     if second_arg != "clean":
         # Add current directory to Python path
         current_dir: str = os.getcwd()
