@@ -3,11 +3,12 @@
 # Imports
 from beet.core.utils import JsonDict
 from beet.library import base
+from beet.toolchain.config import FormatSpecifier
 
 from ..dependencies.bookshelf import BOOKSHELF_MODULES
 
 # Minecraft version specific constants
-MORE_DATA_PACK_FORMATS: dict[tuple[int, ...], int] = {
+MORE_DATA_PACK_FORMATS: dict[tuple[int, ...], FormatSpecifier] = {
 	(1, 21): 48,
 	(1, 21, 0): 48,
 	(1, 21, 1): 48,
@@ -18,10 +19,11 @@ MORE_DATA_PACK_FORMATS: dict[tuple[int, ...], int] = {
 	(1, 21, 6): 80,
 	(1, 21, 7): 81,
 	(1, 21, 8): 81,
-	(1, 21, 9): 88,
-	(1, 21, 10): 88,
+	(1, 21, 9): (88, 0),
+	(1, 21, 10): (88, 0),
+	(2, 0, 0): (1000, 0),
 }
-MORE_ASSETS_PACK_FORMATS: dict[tuple[int, ...], int] = {
+MORE_ASSETS_PACK_FORMATS: dict[tuple[int, ...], FormatSpecifier] = {
 	(1, 21): 34,
 	(1, 21, 0): 34,
 	(1, 21, 1): 34,
@@ -32,8 +34,9 @@ MORE_ASSETS_PACK_FORMATS: dict[tuple[int, ...], int] = {
 	(1, 21, 6): 63,
 	(1, 21, 7): 64,
 	(1, 21, 8): 64,
-	(1, 21, 9): 69,
-	(1, 21, 10): 69,
+	(1, 21, 9): (69, 0),
+	(1, 21, 10): (69, 0),
+	(2, 0, 0): (1000, 0),
 }
 MORE_DATA_VERSIONS: dict[tuple[int, ...], int] = {
 	(1, 21): 3953,
@@ -63,7 +66,9 @@ NO_SILK_TOUCH_DROP: str = "no_silk_touch_drop"			# Key to an item ID that will d
 VANILLA_BLOCK_FOR_ORES: JsonDict = {"id":"minecraft:polished_deepslate", "apply_facing": False}	# Vanilla block that will be used for an optimization tip for ores, don't ask questions
 OVERRIDE_MODEL: str = "override_model"					# Key to a dictionnary that will be used to override the whole model
 PULVERIZING: str = "simplenergy_pulverizing"			# Value of a recipe type, used to generate dusts from ores (used by SimplEnergy)
+AWAKENED_FORGE: str = "stardust_awakened_forge"			# Value of a recipe type, used to craft items in the Awakened Forge (used by Stardust Fragment)
 SMITHED_CRAFTER_COMMAND: str = "smithed_crafter_command"	# Key to a command that will be used in a recipe in the Smithed Crafter library. If not present, the command will be defaulted to a loot table. Ex: {"result":...,SMITHED_CRAFTER_COMMAND: "function your_namespace:calls/smithed_crafter/do_something_else"}
+PAINTING_DATA: str = "painting_data"					# Key to a dict that contains the painting data, like {"author":"","title":"","width":1,"height":1} where author and title defaults to beet config values if not given
 GROWING_SEED: str = "staged_seed"						# Key to a seed that has multiple growth stages, value needs to be a dict like {"texture_basename":"wheat","stages":8,"seconds":600,"planted_on":"stone","loots":[{"id":"minecraft:wheat_seeds","min_count":1,"max_count":3}]} where loots can be either this format or a loot table path "namespace:blocks/loot_table_name"
 WIKI_COMPONENT: str = "wiki_components"					# Key to a text component that will be used to generate the wiki button in the manual
 RESULT_OF_CRAFTING: str = "result_of_crafting"			# Key to a list of recipes to craft the item, ex: "adamantium": {RESULT_OF_CRAFTING: [...]}
@@ -78,6 +83,7 @@ NOT_COMPONENTS: list[str] = [							# Keys that should not be considered as comp
 	NO_SILK_TOUCH_DROP,
 	OVERRIDE_MODEL,
 	SMITHED_CRAFTER_COMMAND,
+	PAINTING_DATA,
 	GROWING_SEED,
 ]
 
@@ -85,6 +91,7 @@ NOT_COMPONENTS: list[str] = [							# Keys that should not be considered as comp
 FACES: tuple[str, ...] = ("down", "up", "north", "south", "west", "east")						# Faces of a block, used for resource pack and blocks orientation
 SIDES: tuple[str, ...] = ("_bottom", "_top", "_front", "_back", "_left", "_right", "_side")	# Sides of a block, used for resource pack
 DOWNLOAD_VANILLA_ASSETS_RAW = "https://raw.githubusercontent.com/edayot/renders/renders/resourcepack/assets/minecraft/textures/render"
+DOWNLOAD_VANILLA_ASSETS_SPECIAL_RAW = "https://raw.githubusercontent.com/edayot/renders/renders-special/resourcepack/assets/minecraft/textures/render"
 DOWNLOAD_VANILLA_ASSETS_SOURCE = "https://github.com/edayot/renders/tree/renders/resourcepack/assets/minecraft/textures/render"
 CUSTOM_BLOCK_HEAD_CUBE_RADIUS: tuple[int, int, int] = (16, 16, 16)	# Size of the region to check around the player when placing a CUSTOM_BLOCK_HEAD
 BLOCKS_WITH_INTERFACES: list[str] = [	# List of blocks that are containers and have an interface
@@ -136,7 +143,7 @@ def official_lib_used(lib: str) -> bool:
 	return is_used
 
 OFFICIAL_LIBS: dict[str, JsonDict] = {
-	"common_signals":		{"version":[0, 1, 1], "name":"Common Signals",					"url":"https://github.com/Stoupy51/CommonSignals",			"is_used": False},
+	"common_signals":		{"version":[0, 2, 0], "name":"Common Signals",					"url":"https://github.com/Stoupy51/CommonSignals",			"is_used": False},
 	"smithed.custom_block":	{"version":[0, 7, 1], "name":"Smithed Custom Block",			"url":"https://wiki.smithed.dev/libraries/custom-block/",	"is_used": False},
 	"smithed.crafter":		{"version":[0, 7, 1], "name":"Smithed Crafter",					"url":"https://wiki.smithed.dev/libraries/crafter/",		"is_used": False},
 	"furnace_nbt_recipes":	{"version":[1, 9, 1], "name":"Furnace NBT Recipes",				"url":"https://github.com/Stoupy51/FurnaceNbtRecipes",		"is_used": False},
