@@ -8,6 +8,7 @@ from ....core.__memory__ import Mem
 from ....core.utils.io import read_function, write_function
 from .context_analyzer import ContextAnalyzer
 from .function_analyzer import FunctionAnalyzer
+from .macro_analyzer import MacroAnalyzer
 from .object import Header
 
 
@@ -37,6 +38,10 @@ def beet_default(ctx: Context):
     context_analyzer = ContextAnalyzer(mcfunctions)
     context_analyzer.analyze_all_contexts()
 
+    # Analyze macro arguments
+    macro_analyzer = MacroAnalyzer(mcfunctions)
+    macro_analyzer.analyze_all_macros()
+
     # Write updated headers to all mcfunction files
     for path, header in mcfunctions.items():
         write_function(path, header.to_str(), overwrite=True)
