@@ -11,11 +11,13 @@ from beet.core.utils import JsonDict, TextComponent
 from PIL import Image
 from stouputils.collections import unique_list
 from stouputils.io import clean_path, relative_path, super_json_dump, super_open
-from stouputils.parallel import colored_for_loop
-from stouputils.print import debug, error, suggestion, warning
+from stouputils.print import colored_for_loop, debug, error, suggestion, warning
+
+from stewbeet.core.definitions_helper.completion import add_private_custom_data_for_namespace
 
 from ...core.__memory__ import Mem
 from ...core.constants import (
+	AWAKENED_FORGE,
 	CATEGORY,
 	CUSTOM_BLOCK_VANILLA,
 	OFFICIAL_LIBS,
@@ -26,15 +28,15 @@ from ...core.constants import (
 	WIKI_COMPONENT,
 )
 from ...core.definitions_helper import add_item_name_and_lore_if_missing
-from ...core.ingredients import CRAFTING_RECIPES_TYPES, ingr_repr, ingr_to_id
+from ...core.ingredients import CRAFTING_RECIPES_TYPES, ingr_repr, ingr_to_id, ingr_to_name
 from ...core.utils.io import super_merge_dict, write_load_file
+from ..custom_recipes.vanilla import VanillaRecipeHandler
 from ..initialize.source_lore_font import find_pack_png
-from ..resource_pack.item_models import (
-	AutoModel,  # Handle new items models (used for the manual and the heavy workbench)
-)
+from ..resource_pack.item_models import AutoModel  # Handle new items models (used for the manual and the heavy workbench)
 from .book_components import get_item_component
 from .book_optimizer import optimize_element, remove_events
 from .craft_content import generate_craft_content
+from .dialog import generate_dialogs
 from .image_utils import (
 	add_border,
 	careful_resize,
@@ -49,11 +51,17 @@ from .other_utils import (
 )
 from .page_font import generate_page_font, generate_wiki_font_for_ingr
 from .shared_import import (
+	AWAKENED_3X3_FONT,
+	AWAKENED_3X4_FONT,
+	AWAKENED_FORGE_STRUCT_FONT,
+	BOOK_FONT,
 	BORDER_COLOR,
 	BORDER_SIZE,
 	FONT_FILE,
 	FURNACE_FONT,
 	HEAVY_WORKBENCH_CATEGORY,
+	HOVER_AWAKENED_3X3_FONT,
+	HOVER_AWAKENED_3X4_FONT,
 	HOVER_EQUIVALENTS,
 	HOVER_FURNACE_FONT,
 	HOVER_MINING_FONT,
@@ -84,6 +92,7 @@ from .shared_import import (
 	get_page_number,
 )
 from .showcase_image import generate_showcase_images
+from .stardust_forge import get_stardust_forge_page
 
 
 # Utility functions
@@ -142,10 +151,14 @@ def routine():
 	os.makedirs(f"{SharedMemory.cache_path}/font/high_res", exist_ok = True)
 	generate_all_iso_renders()
 
+	# Check if there is any awakened forge recipe with 3x3 or 3x4 ingredients
+	has_forge_3x3: bool = any(recipe.get("type") == AWAKENED_FORGE and len(recipe["ingredients"]) <= 9 for data in Mem.definitions.values() for recipe in data.get(RESULT_OF_CRAFTING, []))
+	has_forge_3x4: bool = any(recipe.get("type") == AWAKENED_FORGE and len(recipe["ingredients"]) > 12 for data in Mem.definitions.values() for recipe in data.get(RESULT_OF_CRAFTING, []))
+
 	# Constants
 	FONT = Mem.ctx.project_id + ':' + FONT_FILE
-	MAX_ITEMS_PER_ROW: int = manual_config.get("max_items_per_row", 5)
-	MAX_ROWS_PER_PAGE: int = manual_config.get("max_rows_per_page", 5)
+	MAX_ITEMS_PER_ROW: int = min(6, manual_config.get("max_items_per_row", 5))
+	MAX_ROWS_PER_PAGE: int = min(7, manual_config.get("max_rows_per_page", 5))
 	MAX_ITEMS_PER_PAGE = MAX_ITEMS_PER_ROW * MAX_ROWS_PER_PAGE # (for showing up all items in the categories pages)
 
 	# Calculate left padding for categories pages depending on config['max_items_per_row']: higher the value, lower the padding
@@ -167,12 +180,24 @@ def routine():
 		Mem.ctx.assets[Mem.ctx.project_id].textures["font/stonecutting"] = Texture(source_path=f"{TEMPLATES_PATH}/stonecutting.png")
 		Mem.ctx.assets[Mem.ctx.project_id].textures["font/pulverizing"] = Texture(source_path=f"{TEMPLATES_PATH}/pulverizing.png")
 		Mem.ctx.assets[Mem.ctx.project_id].textures["font/mining"] = Texture(source_path=f"{TEMPLATES_PATH}/mining.png")
+		if has_forge_3x3:
+			Mem.ctx.assets[Mem.ctx.project_id].textures["font/awakened_forge_3x3"] = Texture(source_path=f"{TEMPLATES_PATH}/awakened_forge_3x3.png")
+		if has_forge_3x4:
+			Mem.ctx.assets[Mem.ctx.project_id].textures["font/awakened_forge_3x4"] = Texture(source_path=f"{TEMPLATES_PATH}/awakened_forge_3x4.png")
+		if has_forge_3x3 or has_forge_3x4:
+			Mem.ctx.assets[Mem.ctx.project_id].textures["font/awakened_forge_1"] = Texture(source_path=f"{TEMPLATES_PATH}/awakened_forge_1.png")
+			Mem.ctx.assets[Mem.ctx.project_id].textures["font/awakened_forge_2"] = Texture(source_path=f"{TEMPLATES_PATH}/awakened_forge_2.png")
+	if SharedMemory.use_dialog > 0:
+		Mem.ctx.assets[Mem.ctx.project_id].textures["font/book"] = Texture(source_path=f"{TEMPLATES_PATH}/book.png")
+
+	# Prepare category padding if dialog
+	category_padding: list[str] = [VERY_SMALL_NONE_FONT] if SharedMemory.use_dialog > 0 else []
 
 	# If the manual cache is enabled and we have a cache file, load it
 	cache_pages: bool = manual_config.get("cache_pages", False)
 	if cache_pages and json_dump_path and os.path.exists(json_dump_path) and os.path.exists(f"{SharedMemory.cache_path}/font/manual.json"):
 		with super_open(json_dump_path, "r") as f:
-			book_content: list[TextComponent] = json.load(f)
+			book_content: list[list[TextComponent]] = json.load(f)
 
 	# Else, generate all
 	else:
@@ -236,7 +261,7 @@ def routine():
 			SharedMemory.manual_pages.append({"number": i, "name": item, "raw_data": data, "type": "item"})
 
 		# Encode pages
-		book_content: list[TextComponent] = []
+		book_content: list[list[TextComponent]] = []
 		os.makedirs(f"{SharedMemory.cache_path}/font/category", exist_ok=True)
 		simple_case = load_simple_case_no_border(SharedMemory.high_resolution)	# Load the simple case image for later use in categories pages
 		def encode_page(page: JsonDict):
@@ -265,6 +290,7 @@ def routine():
 				line: list[TextComponent] = []
 
 				# For each item in the category, get its page number and texture, then add it to the image
+				max_items_reached: bool = False
 				for item in raw_data:
 
 					# Get item texture
@@ -288,30 +314,35 @@ def routine():
 					x += simple_case.size[0]
 
 					# Add the click_event part to the line and add the 2 times the line if enough items
-					component = get_item_component(item, only_those_components=["item_name", "custom_name"])
+					component = get_item_component(item, only_those_components=[x for x in SharedMemory.components_to_include if x != "lore"])
 					component["text"] = MEDIUM_NONE_FONT if not SharedMemory.high_resolution else high_res_font
 					line.append(component)
 					if len(line) == MAX_ITEMS_PER_ROW:
+						max_items_reached = True
 						line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
-						content += deepcopy(line)
+						content.extend(deepcopy(line))
+						content.extend(category_padding)
 						for i in range(1, len(line)):
 							selected = line[-i]
 							if isinstance(selected, dict):
 								selected["text"] = MEDIUM_NONE_FONT
-						content += ["\n", *line, "\n"]
+						content.extend(["\n", *line, *category_padding, "\n"])
 						line = []
 						x = 2
 						y += simple_case.size[1]
 
 				# If remaining items in the line, add them
 				if len(line) > 0:
+					if SharedMemory.use_dialog > 0 and max_items_reached:
+						line.append(MEDIUM_NONE_FONT * max(0, MAX_ITEMS_PER_ROW - len(line)))
 					line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
-					content += deepcopy(line)
+					content.extend(deepcopy(line))
+					content.extend(category_padding)
 					for i in range(1, len(line)):
 						selected = line[-i]
 						if isinstance(selected, dict):
 							selected["text"] = MEDIUM_NONE_FONT
-					content += ["\n", *line, "\n"]
+					content.extend(["\n", *line, *category_padding, "\n"])
 
 				# Add the 2 pixels border
 				is_rectangle_shape = len(raw_data) % MAX_ITEMS_PER_ROW == 0
@@ -410,9 +441,14 @@ def routine():
 						component["text"] *= 2
 						content.append({"text": "", "font": FONT, "color": "white"})	# Make default font for every next component
 						content.append({"text": titled, "font": "minecraft:default", "color": "black", "underlined": True})
-						content.append(MEDIUM_NONE_FONT * 2 + page_font + "\n")
+						if not SharedMemory.use_dialog > 0:
+							content.append(MEDIUM_NONE_FONT * 2 + page_font + "\n")
+						else:
+							content.append(page_font + "\n")
+
 						for _ in range(4):
-							content.append(MEDIUM_NONE_FONT * 2)
+							if not SharedMemory.use_dialog > 0:
+								content.append(MEDIUM_NONE_FONT * 2)
 							content.append(component)
 							content.append("\n")
 						content_added = True
@@ -427,13 +463,25 @@ def routine():
 					])
 				else:
 					if raw_data.get(WIKI_COMPONENT):
+						found_event: JsonDict | None = None
+						wiki_component: TextComponent = raw_data[WIKI_COMPONENT]
+						if isinstance(wiki_component, dict) and "click_event" in wiki_component:
+							found_event = wiki_component["click_event"]
+						elif isinstance(wiki_component, list):
+							for comp in wiki_component:
+								if isinstance(comp, dict) and "click_event" in comp:
+									found_event = cast(JsonDict, comp["click_event"])
+									break
+
 						info_buttons.append({
 							"text": WIKI_INFO_FONT + VERY_SMALL_NONE_FONT * 2,
 							"hover_event": {
 								"action": "show_text",
-								"value": raw_data[WIKI_COMPONENT]
+								"value": wiki_component
 							}
 						})
+						if found_event:
+							info_buttons[-1]["click_event"] = found_event
 
 					# For each craft (except smelting dupes),
 					previous_result: Any = None
@@ -441,6 +489,12 @@ def routine():
 						if craft["type"] == "crafting_shapeless":
 							craft = convert_shapeless_to_shaped(craft)
 
+						# Skip if result is same as previous result
+						current_result: Any | None = craft.get("result")
+						if current_result and current_result == previous_result and craft["type"] != "mining":
+							continue
+						previous_result = current_result
+
 						# Get breaklines
 						breaklines = 3
 						if "shape" in craft:
@@ -452,7 +506,7 @@ def routine():
 							hover_text: list[TextComponent] = [{"text":""}]
 							hover_text.append({"text": craft_font + "\n\n" * breaklines, "font": FONT, "color": "white"})
 						else:
-							craft_content: list[TextComponent] = generate_craft_content(craft, name, "")
+							craft_content: list[TextComponent] = generate_craft_content(craft, name, "", in_lore=True)
 							craft_content = [craft_content[0]] + craft_content[2:]	# Remove craft title
 							remove_events(craft_content)
 							for k, v in HOVER_EQUIVALENTS.items():
@@ -473,23 +527,24 @@ def routine():
 								"stonecutting": "Stonecutting",
 								"smithing_transform": "Smithing Transform",
 								"smithing_trim": "Smithing Trim",
-								PULVERIZING: "SimplEnergy Pulverizing"
+								PULVERIZING: "(SimplEnergy) Pulverizing",
+								AWAKENED_FORGE: "(Stardust Fragment) Awakened Forge",
 							}
 							recipe_title = recipe_type_names.get(craft["type"], craft["type"].replace("_", " ").title())
-							hover_text.insert(0, {"text": f"{recipe_title}\n", "color": "yellow"})
+							hover_text.append({"text": f"\n{recipe_title}", "color": "yellow"})
 
 						# Append ingredients
 						if craft["type"] == "mining":
 							# For mining recipes, show what is being mined and what it drops
-							ore_name = ingr_to_id(craft["ingredient"], False).replace("_", " ").title()
-							result_name = ingr_to_id(craft["result"], False).replace("_", " ").title()
+							ore_name = ingr_to_name(craft["ingredient"])
+							result_name = ingr_to_name(craft["result"])
 							hover_text.append({"text": "\n- Mine: ", "color": "gray"})
 							hover_text.append({"text": ore_name, "color": "gray"})
 							result_count = craft.get("result_count", "1")
 							hover_text.append({"text": f"\n- Drops: x{result_count} ", "color": "gray"})
 							hover_text.append({"text": result_name, "color": "gray"})
 						elif craft.get("ingredient"):
-							id = ingr_to_id(craft["ingredient"], False).replace("_", " ").title()
+							id = ingr_to_name(craft["ingredient"])
 							hover_text.append({"text": "\n- x1 ", "color": "gray"})
 							hover_text.append({"text": id, "color": "gray"})
 						elif craft.get("ingredients"):
@@ -497,7 +552,7 @@ def routine():
 							# If it's a shaped crafting
 							if isinstance(craft["ingredients"], dict):
 								for k, v in craft["ingredients"].items():
-									id = ingr_to_id(v, False).replace("_", " ").title()
+									id = ingr_to_name(v)
 									count = sum([line.count(k) for line in craft["shape"]])
 									hover_text.append({"text": f"\n- x{count} ", "color": "gray"})
 									hover_text.append({"text": id, "color": "gray"})
@@ -506,25 +561,25 @@ def routine():
 							elif isinstance(craft["ingredients"], list):
 								ids: dict[str, int] = {}	# {id: count}
 								for ingr in craft["ingredients"]:
-									id = ingr_to_id(ingr, False).replace("_", " ").title()
+									id = ingr_to_name(ingr)
 									if id not in ids:
 										ids[id] = 0
-									ids[id] += 1
+									ids[id] += ingr.get("count", 1)
 								for id, count in ids.items():
 									hover_text.append({"text": f"\n- x{count} ", "color": "gray"})
 									hover_text.append({"text": id, "color": "gray"})
 						else:
 							# Smithing crafts
 							if craft.get("base"):
-								id = ingr_to_id(craft["base"], False).replace("_", " ").title()
+								id = ingr_to_name(craft["base"])
 								hover_text.append({"text": "\n- Base: ", "color": "gray"})
 								hover_text.append({"text": id, "color": "gray"})
 							if craft.get("template"):
-								id = ingr_to_id(craft["template"], False).replace("_", " ").title()
+								id = ingr_to_name(craft["template"])
 								hover_text.append({"text": "\n- Template: ", "color": "gray"})
 								hover_text.append({"text": id, "color": "gray"})
 							if craft.get("addition"):
-								id = ingr_to_id(craft["addition"], False).replace("_", " ").title()
+								id = ingr_to_name(craft["addition"])
 								hover_text.append({"text": "\n- Addition: ", "color": "gray"})
 								hover_text.append({"text": id, "color": "gray"})
 							if craft.get("pattern"):
@@ -532,12 +587,6 @@ def routine():
 								hover_text.append({"text": "\n- Pattern: ", "color": "gray"})
 								hover_text.append({"text": pattern_name, "color": "gray"})
 
-						# Skip if result is same as previous result
-						current_result: Any | None = craft.get("result")
-						if current_result and current_result == previous_result and craft["type"] != "mining":
-							continue
-						previous_result = current_result
-
 						# Add the craft to the content
 						result_or_ingredient = WIKI_RESULT_OF_CRAFT_FONT if "result" not in craft else generate_wiki_font_for_ingr(name, craft)
 						info_buttons.append({
@@ -580,6 +629,8 @@ def routine():
 
 					# Retrieve buttons limit depending on number of lines in the content (more lines = less buttons)
 					buttons_limit: int = 20 if str(content).count("\\n") <= 6 else 15
+					if SharedMemory.use_dialog > 0:
+						buttons_limit = 42	# Higher limit
 
 					# If too many buttons, remove all the blue ones (blue_craft) except the last one
 					if len(info_buttons) > buttons_limit:
@@ -595,7 +646,7 @@ def routine():
 
 						# If there are more than buttons_limit buttons, remove lowest priority ones until there is buttons_limit left
 						while len(info_buttons) > buttons_limit:
-							lowest_priority: int = min(info_buttons[::-1], key=lambda x: x.get("priority", 1))
+							lowest_priority: JsonDict = min(info_buttons[::-1], key=lambda x: x.get("priority", 1))
 							info_buttons.remove(lowest_priority)
 
 						# Keep only the last buttons_limit buttons (maximum that can be displayed with 5 per line and 4 lines)
@@ -612,25 +663,26 @@ def routine():
 					content.append("\n")
 
 					last_i = 0
+					buttons_per_line = 5 if not SharedMemory.use_dialog > 0 else 6
 					for i, button in enumerate(info_buttons):
 						last_i = i
 						# Duplicate line and add breakline
-						if i % 5 == 0 and i != 0:
+						if i % buttons_per_line == 0 and i != 0:
 							# Remove VERY_SMALL_NONE_FONT from last button to prevent automatic break line
 							last_content = cast(JsonDict, content[-1])
 							last_content["text"] = last_content["text"].replace(VERY_SMALL_NONE_FONT, "")
 
-							# Re-add last 5 buttons (for good hover_event) but we replace the wiki font by the small font
-							content += ["\n"] + [cast(JsonDict, x).copy() for x in content[-5:]]
-							for j in range(5):
-								selected_content = cast(JsonDict, content[-5 + j])
-								selected_content["text"] = WIKI_NONE_FONT + VERY_SMALL_NONE_FONT * (2 if j != 4 else 0)
+							# Re-add last buttons (for good hover_event) but we replace the wiki font by the small font
+							content += ["\n"] + [cast(JsonDict, x).copy() for x in content[-buttons_per_line:]]
+							for j in range(buttons_per_line):
+								selected_content = cast(JsonDict, content[-buttons_per_line + j])
+								selected_content["text"] = WIKI_NONE_FONT + VERY_SMALL_NONE_FONT * (2 if j != (buttons_per_line - 1) else 0)
 							content.append("\n")
 						content.append(button)
 
 					# Duplicate the last line if not done yet
-					if last_i % 5 != 0 or last_i == 0:
-						last_i = last_i % 5 + 1
+					if last_i % buttons_per_line != 0 or last_i == 0:
+						last_i = last_i % buttons_per_line + 1
 
 						# Remove VERY_SMALL_NONE_FONT from last button to prevent automatic break line
 						last_content = cast(JsonDict, content[-1])
@@ -639,7 +691,7 @@ def routine():
 						content += ["\n"] + [cast(JsonDict, x).copy() for x in content[-last_i:]]
 						for j in range(last_i):
 							selected_content = cast(JsonDict, content[-last_i + j])
-							selected_content["text"] = WIKI_NONE_FONT + VERY_SMALL_NONE_FONT * (2 if j != 4 else 0)
+							selected_content["text"] = WIKI_NONE_FONT + VERY_SMALL_NONE_FONT * (2 if j != (last_i - 1) else 0)
 
 			# Add page to the book
 			book_content.append(content)
@@ -664,6 +716,7 @@ def routine():
 		line: list[TextComponent] = []
 
 		# For each item in the category, get its page number and texture, then add it to the image
+		max_items_reached: bool = False
 		for page in SharedMemory.manual_pages:
 			if page["type"] == CATEGORY:
 				item = page["raw_data"][0]
@@ -698,24 +751,28 @@ def routine():
 					component["text"] = high_res_font
 				line.append(component)
 				if len(line) == MAX_ITEMS_PER_ROW:
+					max_items_reached = True
 					line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
-					content += [*deepcopy(line), "\n"]
+					content += [*deepcopy(line), *category_padding, "\n"]
 					for i in range(1, len(line)):
 						selected = cast(JsonDict, line[-i])
 						selected["text"] = MEDIUM_NONE_FONT
-					content += [*line, "\n"]
+					content += [*line, *category_padding, "\n"]
 					line = []
 					x = 2
 					y += simple_case.size[1]
 
 		# If remaining items in the line, add them
 		if len(line) > 0:
+			if SharedMemory.use_dialog > 0 and max_items_reached:
+				line.append(MEDIUM_NONE_FONT * max(0, 5 - len(line)))
 			line.insert(0, SMALL_NONE_FONT * LEFT_PADDING)
-			content += [*deepcopy(line), "\n"]
+			content += [*deepcopy(line), *category_padding, "\n"]
 			for i in range(1, len(line)):
-				selected = cast(JsonDict, line[-i])
-				selected["text"] = MEDIUM_NONE_FONT
-			content += [*line, "\n"]
+				selected = cast(JsonDict | str, line[-i])
+				if isinstance(selected, dict):
+					selected["text"] = MEDIUM_NONE_FONT
+			content += [*line, *category_padding, "\n"]
 
 		# Add the 2 pixels border
 		is_rectangle_shape = len(categories_pages) % MAX_ITEMS_PER_ROW == 0
@@ -731,7 +788,7 @@ def routine():
 		page_font = get_page_font(0)
 		SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/page/_logo.png", "ascent": 0, "height": 40, "chars": [page_font]})
 		intro_content.append({"text": manual_name + "\n", "underlined": True})
-		intro_content.append({"text": MEDIUM_NONE_FONT * 2 + page_font, "font": FONT, "color": "white"})
+		intro_content.append({"text": MEDIUM_NONE_FONT * (0 if SharedMemory.use_dialog > 0 else 2) + page_font, "font": FONT, "color": "white"})
 
 		# Create the image and load Minecraft font
 
@@ -743,10 +800,7 @@ def routine():
 		# Write the introduction text
 		intro_content.append({"text": "\n" * 6})
 		first_page_config: TextComponent = manual_config.get('first_page_text', "")
-		if isinstance(first_page_config, list):
-			intro_content.extend(first_page_config)
-		else:
-			intro_content.append(first_page_config)
+		intro_content.append([{"text": "", "font": "minecraft:default", "color": "black"}, first_page_config])
 
 		# Save image and insert in the manual pages
 		logo.save(f"{SharedMemory.cache_path}/font/page/_logo.png")
@@ -754,7 +808,19 @@ def routine():
 
 		## Optimize the book size
 		book_content_deepcopy: list[TextComponent] = deepcopy(book_content)	# Deepcopy to avoid sharing same components (such as click_event)
-		book_content = list(optimize_element(book_content_deepcopy))
+		book_content = cast(list[list[TextComponent]], list(optimize_element(book_content_deepcopy)))
+
+		## Insert at 2nd page the awakened forge
+		if has_forge_3x3 or has_forge_3x4:
+			book_content.insert(1, get_stardust_forge_page())
+
+			# Increase every change_page click event by 1
+			for page in book_content:
+				for component in page:
+					if isinstance(component, dict):
+						if "click_event" in component and component["click_event"].get("action") == "change_page":
+							current_value: int = int(component["click_event"]["page"])
+							component["click_event"]["page"] = current_value + 1
 
 		## Insert at 2nd page the heavy workbench
 		if "heavy_workbench" in Mem.definitions:
@@ -765,7 +831,6 @@ def routine():
 			for page in book_content:
 				for component in page:
 					if isinstance(component, dict):
-						component = cast(JsonDict, component)
 						if "click_event" in component and component["click_event"].get("action") == "change_page":
 							current_value: int = int(component["click_event"]["page"])
 							component["click_event"]["page"] = current_value + 1
@@ -794,6 +859,17 @@ def routine():
 			SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/stonecutting.png", "ascent": -3, "height": 58, "chars": [HOVER_STONECUTTING_FONT]})
 			SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/pulverizing.png", "ascent": -3, "height": 58, "chars": [HOVER_PULVERIZING_FONT]})
 			SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/mining.png", "ascent": -3, "height": 58, "chars": [HOVER_MINING_FONT]})
+			if has_forge_3x3:
+				SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/awakened_forge_3x3.png", "ascent": 9, "height": 74, "chars": [AWAKENED_3X3_FONT]})
+				SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/awakened_forge_3x3.png", "ascent": 4, "height": 74, "chars": [HOVER_AWAKENED_3X3_FONT]})
+			if has_forge_3x4:
+				SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/awakened_forge_3x4.png", "ascent": 9, "height": 74, "chars": [AWAKENED_3X4_FONT]})
+				SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/awakened_forge_3x4.png", "ascent": 4, "height": 74, "chars": [HOVER_AWAKENED_3X4_FONT]})
+			if has_forge_3x3 or has_forge_3x4:
+				SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/awakened_forge_1.png", "ascent": 8, "height": 56, "chars": [AWAKENED_FORGE_STRUCT_FONT[0]]})
+				SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/awakened_forge_2.png", "ascent": 8, "height": 56, "chars": [AWAKENED_FORGE_STRUCT_FONT[1]]})
+		if SharedMemory.use_dialog > 0:
+			SharedMemory.font_providers.append({"type":"bitmap","file":f"{Mem.ctx.project_id}:font/book.png", "ascent": 25, "height": 300, "chars": [BOOK_FONT]})
 		fonts = {"providers": SharedMemory.font_providers}
 		with super_open(f"{SharedMemory.cache_path}/font/manual.json", "w") as f:
 			f.write(super_json_dump(fonts))
@@ -830,36 +906,48 @@ def routine():
 			if len(fp["chars"]) < 1 or (len(fp["chars"]) == 1 and not fp["chars"][0]):
 				error(f"Font provider '{path}' has no chars")
 
-	# Finally, prepend the manual to the definitions
-	manual_already_exists: bool = "manual" in Mem.definitions
-	manual_definitions: JsonDict = {
-		"manual": {
-			"id": "minecraft:written_book",
-			"written_book_content": {
-				"title": manual_name,
-				"author": Mem.ctx.project_author,
-				"pages": book_content,
-			},
-			"item_model": f"{Mem.ctx.project_id}:manual",
-			"item_name": manual_name,
-			"enchantment_glint_override": False,
-			"max_stack_size": 1
-		}
-	}
-	if not Mem.definitions.get("manual"):
-		Mem.definitions["manual"] = manual_definitions["manual"]
-	else:
-		Mem.definitions["manual"] = super_merge_dict(manual_definitions["manual"], Mem.definitions["manual"])
-	add_item_name_and_lore_if_missing(black_list=[item for item in Mem.definitions if item != "manual"])
+	# Generate dialog files if needed
+	if SharedMemory.use_dialog > 0:
+		generate_dialogs(book_content)
 
-	# Add the model to the resource pack if it doesn't already exist
-	if not manual_already_exists:
-		textures_folder: str = relative_path(Mem.ctx.meta.get("stewbeet", {}).get("textures_folder", ""))
-		textures: dict[str, str] = {
-			clean_path(str(p)).split("/")[-1]: relative_path(str(p))
-			for p in Path(textures_folder).rglob("*.png")
+	# Finally, prepend the manual to the definitions
+	if SharedMemory.use_dialog != 2:
+		manual_already_exists: bool = "manual" in Mem.definitions
+		manual_definitions: JsonDict = {
+			"manual": {
+				"id": "minecraft:written_book",
+				"written_book_content": {
+					"title": manual_name,
+					"author": Mem.ctx.project_author,
+					"pages": book_content,
+				},
+				"item_model": f"{Mem.ctx.project_id}:manual",
+				"item_name": manual_name,
+				"enchantment_glint_override": False,
+				"max_stack_size": 16
+			}
 		}
-		AutoModel.from_definitions("manual", Mem.definitions["manual"], textures).process()
+		if SharedMemory.use_dialog > 0:
+			del manual_definitions["manual"]["written_book_content"]
+			manual_definitions["manual"]["lore"] = [{"text": f"by {Mem.ctx.project_author}", "color": "gray", "italic": False}]
+		if not Mem.definitions.get("manual"):
+			Mem.definitions["manual"] = manual_definitions["manual"]
+		else:
+			Mem.definitions["manual"] = super_merge_dict(manual_definitions["manual"], Mem.definitions["manual"])
+		add_item_name_and_lore_if_missing(black_list=[item for item in Mem.definitions if item != "manual"])
+		add_private_custom_data_for_namespace(black_list=[item for item in Mem.definitions if item != "manual"])
+
+		# Add the model to the resource pack if it doesn't already exist
+		if not manual_already_exists:
+			textures_folder: str = relative_path(Mem.ctx.meta.get("stewbeet", {}).get("textures_folder", ""))
+			textures: dict[str, str] = {
+				clean_path(str(p)).split("/")[-1]: relative_path(str(p))
+				for p in Path(textures_folder).rglob("*.png")
+			}
+			AutoModel.from_definitions("manual", Mem.definitions["manual"], textures).process()
+
+		# Repair the recipes for the manual
+		VanillaRecipeHandler().generate_recipes(override=["manual"])
 
 	# Remove the heavy workbench from the definitions
 	if OFFICIAL_LIBS["smithed.crafter"]["is_used"]:
@@ -868,7 +956,6 @@ def routine():
 		del Mem.ctx.assets[Mem.ctx.project_id].models["item/heavy_workbench"]
 		del Mem.ctx.assets[Mem.ctx.project_id].item_models["heavy_workbench"]
 
-
 	# Register of the manual in the universal manual
 	first_page: str = json.dumps(book_content[0], ensure_ascii=False)
 	for r in [("\\n", "\\\\n"), (', "underlined": true','')]:
