@@ -5,16 +5,30 @@ from __future__ import annotations
 import os
 from collections.abc import Iterable
 
-from beet import ItemModel, Model, Texture
+from beet import ItemModel, Model
 from beet.core.utils import JsonDict
 from stouputils.decorators import LogLevels, handle_error, simple_cache
-from stouputils.io import super_json_dump, super_json_load
+from stouputils.io import super_json_dump
+from stouputils.print import error
 
 from ....core.__memory__ import Mem
-from ....core.constants import CUSTOM_BLOCK_VANILLA, CUSTOM_ITEM_VANILLA, OVERRIDE_MODEL
-from ....core.utils.io import set_json_encoder
+from ....core.constants import CUSTOM_BLOCK_VANILLA, CUSTOM_ITEM_VANILLA, GROWING_SEED, OVERRIDE_MODEL
+from ....core.utils.io import set_json_encoder, texture_mcmeta
 
 
+# Utility function
+def to_atlas(texture: str) -> str:
+	""" Convert a texture path to its atlas sprite path.
+
+	Args:
+		texture (str): The original texture path.
+
+	Returns:
+		str: The converted atlas sprite path.
+	"""
+	return f"{Mem.ctx.project_id}:atlas/" + "/".join(texture.split("/")[1:])  # Remove 'minecraft:block/' prefix
+
+# Class
 class AutoModel:
 	""" Class to handle item model processing.
 
@@ -26,6 +40,7 @@ class AutoModel:
 		namespace       (str):            The namespace of the item model.
 		block_or_item   (str):            Whether this is a block or item model.
 		used_textures   (set[str]):       Set of used textures.
+		used_minecraft_textures (set[str]): Set of used Minecraft textures.
 		source_textures (dict[str, str]): Dictionary of source textures.
 		ignore_textures (bool):           Whether to ignore texture-related errors.
 	"""
@@ -42,9 +57,10 @@ class AutoModel:
 		"""
 		self.item_name: str = item_name
 		self.data: JsonDict = data
-		self.namespace: str = Mem.ctx.project_id
+		self.ns: str = Mem.ctx.project_id
 		self.block_or_item: str = "item"
 		self.used_textures: set[str] = set()
+		self.used_minecraft_textures: set[str] = set()
 		self.source_textures: dict[str, str] = source_textures
 		self.ignore_textures: bool = ignore_textures
 
@@ -142,16 +158,74 @@ class AutoModel:
 						same_folder_variants.append(variant)
 		return same_folder_variants
 
+	def handle_growing_seeds(self) -> None:
+		""" Handle growing seeds by adding growth stage models and textures. """
+		# Retrieve growing seed data
+		growing_seed_data: JsonDict = self.data[GROWING_SEED]
+		texture_basename: str = growing_seed_data.get("texture_basename", self.item_name)
+		planted_on: str = growing_seed_data["planted_on"]
+		if planted_on == "magma_block":
+			planted_on = "magma"
+
+		# Find all stage textures and order them by stage number
+		stage_textures: dict[str, str] = dict(sorted({
+				k.replace(".png", ""): v for k, v in self.source_textures.items()
+				if os.path.basename(k).startswith(f"{texture_basename}_stage_")
+			}.items(),
+			key=lambda item: int(item[0].split("_")[-1])
+		))
+
+		# For each stage, create the model and add the texture
+		for i in range(len(stage_textures)):
+			stage_texture_name: str = f"{texture_basename}_stage_{i}"
+			if stage_texture_name not in stage_textures:
+				if not self.ignore_textures:
+					error(f"Missing texture for growing seed stage: '{stage_texture_name}.png'")
+				continue
+
+			# Create model for this stage
+			self.used_minecraft_textures.add(f"minecraft:block/{planted_on}")
+			stage_model: JsonDict = {
+				"textures": {
+					"1": to_atlas(f"minecraft:block/{planted_on}"),
+					"2": f"{self.ns}:item/seeds/{stage_texture_name}",
+					"particle": to_atlas(f"minecraft:block/{planted_on}")
+				},
+				"elements": [
+					{"name":"seed","from":[0,1,4],"to":[16,17,4],"faces":{"north":{"uv":[0,0,16,16],"texture":"#2"},"south":{"uv":[16,0,0,16],"texture":"#2"}}},
+					{"name":"seed","from":[12,1,0],"to":[12,17,16],"faces":{"east":{"uv":[0,0,16,16],"texture":"#2"},"west":{"uv":[16,0,0,16],"texture":"#2"}}},
+					{"name":"seed","from":[0,1,12],"to":[16,17,12],"faces":{"north":{"uv":[16,0,0,16],"texture":"#2"},"south":{"uv":[0,0,16,16],"texture":"#2"}}},
+					{"name":"seed","from":[4,1,0],"to":[4,17,16],"faces":{"east":{"uv":[16,0,0,16],"texture":"#2"},"west":{"uv":[0,0,16,16],"texture":"#2"}}},
+					{"name":"base","from":[0,0,0],"to":[16,1.05,16],"faces":{"north":{"uv":[0,0,16,1],"texture":"#1"},"east":{"uv":[0,0,16,1],"texture":"#1"},"south":{"uv":[0,0,16,1],"texture":"#1"},"west":{"uv":[0,0,16,1],"texture":"#1"},"up":{"uv":[0,0,16,16],"texture":"#1"},"down":{"uv":[0,0,16,16],"texture":"#1"}}}
+				]
+			}
+
+			# Add the model to assets and create item model
+			Mem.ctx.assets[self.ns].models[f"item/seeds/{stage_texture_name}"] = set_json_encoder(Model(stage_model), max_level=4)
+			items_model = {"model": {"type": "minecraft:model", "model": f"{self.ns}:item/seeds/{stage_texture_name}"}}
+			Mem.ctx.assets[self.ns].item_models[f"seeds/{stage_texture_name}"] = set_json_encoder(ItemModel(items_model), max_level=4)
+
+			# Add the texture to assets
+			Mem.ctx.assets[self.ns].textures[f"item/seeds/{stage_texture_name}"] = texture_mcmeta(stage_textures[stage_texture_name])
+
 	@handle_error(exceptions=ValueError, error_log=LogLevels.ERROR_TRACEBACK)
-	def process(self) -> None:
-		""" Process the item model. """
+	def process(self) -> set[str]:
+		""" Process the item model.
+
+		Returns:
+			set[str]: Set of blocks textures to be added to the items atlas.
+		"""
+		# If the item is a growing seed, handle it
+		if GROWING_SEED in self.data:
+			self.handle_growing_seeds()
+
 		# If no item model, return
 		if not self.data.get("item_model"):
-			return
+			return set()
 
 		# If item_model is already processed, return
 		if self.data["item_model"] in Mem.ctx.meta["stewbeet"]["rendered_item_models"]:
-			return
+			return set()
 
 		# Initialize variables
 		if (self.data.get("id") == CUSTOM_BLOCK_VANILLA or
@@ -164,9 +238,12 @@ class AutoModel:
 		exclude_textures: bool = "textures" in overrides and overrides.get("textures") is None
 
 		# Get powered states (if any)
-		powered = [""]
+		powered: list[str] = [""]
+		count: int = self.item_name.count("_")
 		for texture_name in self.source_textures:
-			if texture_name.startswith(self.item_name) and texture_name.endswith("_on.png"):
+			texture_count: int = texture_name.count("_")
+			# Only consider textures with similar underscore count to avoid false positives
+			if texture_name.startswith(self.item_name) and texture_name.endswith("_on.png") and abs(texture_count - count) <= 2:
 				powered = ["", "_on"]
 
 		# Debug
@@ -183,6 +260,7 @@ class AutoModel:
 			all_variants: list[str] = [
 				x.replace(".png", "") for x in self.source_textures
 				if os.path.basename(x).startswith(self.item_name)
+				and abs(x.count("_") - self.item_name.count("_")) <= 2  # Allow for up to 2 extra underscores. Preventing "awakened_stardust.png" to match "awakened_stardust_furnace_generator_on.png"
 			]
 			# Filter to only include variants in the same folder
 			variants: list[str] = self.get_same_folder_variants(all_variants)
@@ -195,7 +273,7 @@ class AutoModel:
 					# Check in which variants state we are
 					variants_without_on = [x for x in variants if "_on" not in x]
 					if not exclude_textures and len(variants_without_on) == 1:
-						content["textures"]["all"] = f"{self.namespace}:item/" + self.get_powered_texture(variants, "", on_off)
+						content["textures"]["all"] = f"{self.ns}:item/" + self.get_powered_texture(variants, "", on_off)
 					elif not exclude_textures:
 						# Prepare models to check
 						cake = ["bottom", "side", "top", "inner"]
@@ -206,28 +284,34 @@ class AutoModel:
 							content["parent"] = "block/cake"
 							for side in cake:
 								texture_key = side.replace("inner", "inside")
-								texture_path = f"{self.namespace}:item/" + self.get_powered_texture(variants, side, on_off)
+								texture_path = f"{self.ns}:item/" + self.get_powered_texture(variants, side, on_off)
 								content["textures"][texture_key] = texture_path
 
 							# Generate 6 models for each cake slice
 							for i in range(1, 7):
 								name: str = f"{self.item_name}_slice{i}"
 								slice_content: JsonDict = {"parent": f"block/cake_slice{i}", "textures": content["textures"]}
-								Mem.ctx.assets[f"{self.namespace}:item/{name}{on_off}"] = Model(super_json_dump(slice_content, max_level=4))						# Check cube_bottom_top model
+								Mem.ctx.assets[self.ns].models[f"item/{name}{on_off}"] = set_json_encoder(Model(slice_content), max_level=4)
+
+						# Check cube_bottom_top model
 						elif self.model_in_variants(cube_bottom_top, variants):
 							content["parent"] = "block/cube_bottom_top"
 							for side in cube_bottom_top:
-								texture_path = f"{self.namespace}:item/" + self.get_powered_texture(variants, side, on_off)
-								content["textures"][side] = texture_path						# Check orientable model
+								texture_path = f"{self.ns}:item/" + self.get_powered_texture(variants, side, on_off)
+								content["textures"][side] = texture_path
+
+						# Check orientable model
 						elif self.model_in_variants(orientable, variants):
 							content["parent"] = "block/orientable"
 							for side in orientable:
-								texture_path = f"{self.namespace}:item/" + self.get_powered_texture(variants, side, on_off)
-								content["textures"][side] = texture_path						# Check cube_column model
+								texture_path = f"{self.ns}:item/" + self.get_powered_texture(variants, side, on_off)
+								content["textures"][side] = texture_path
+
+						# Check cube_column model
 						elif self.model_in_variants(cube_column, variants):
 							content["parent"] = "block/cube_column"
 							for side in cube_column:
-								texture_path = f"{self.namespace}:item/" + self.get_powered_texture(variants, side, on_off)
+								texture_path = f"{self.ns}:item/" + self.get_powered_texture(variants, side, on_off)
 								content["textures"][side] = texture_path
 
 						# Else, if there are no textures override, show error
@@ -250,14 +334,14 @@ class AutoModel:
 					# Get parent
 					parent = "item/generated"
 					data_id: str = self.data["id"]
-					if data_id != CUSTOM_ITEM_VANILLA:
+					if data_id != CUSTOM_ITEM_VANILLA and "elements" not in self.data.get(OVERRIDE_MODEL, {}):
 						parent = data_id.replace(':', ":item/")
 
 					# Get textures
 					if exclude_textures:
 						content = {"parent": parent}
 					else:
-						textures = {"layer0": f"{self.namespace}:item/{self.item_name}{on_off}"}
+						textures = {"layer0": f"{self.ns}:item/{self.item_name}{on_off}"}
 						content = {"parent": parent, "textures": textures}
 					data_id = data_id.replace("minecraft:", "")
 
@@ -267,7 +351,7 @@ class AutoModel:
 
 					# If there is a "_overlay" texture, make it as layer1
 					if not exclude_textures and f"{self.item_name}_overlay" in variants:
-						content["textures"]["layer1"] = f"{self.namespace}:item/{self.item_name}_overlay"
+						content["textures"]["layer1"] = f"{self.ns}:item/{self.item_name}_overlay"
 
 					# Check for bow pulling textures
 					elif not exclude_textures and data_id.endswith("bow"):
@@ -281,14 +365,14 @@ class AutoModel:
 								"type": "minecraft:condition",
 								"on_false": {
 									"type": "minecraft:model",
-									"model": f"{self.namespace}:item/{self.item_name}"
+									"model": f"{self.ns}:item/{self.item_name}"
 								},
 								"on_true": {
 									"type": "minecraft:range_dispatch",
 									"entries": [],
 									"fallback": {
 										"type": "minecraft:model",
-										"model": f"{self.namespace}:item/{self.item_name}_pulling_0"
+										"model": f"{self.ns}:item/{self.item_name}_pulling_0"
 									},
 									"property": "minecraft:use_duration",
 									"scale": 0.05
@@ -298,16 +382,19 @@ class AutoModel:
 
 							# Add override for each pulling state
 							for i, variant in enumerate(sorted_pull_variants):
-								pull_content: JsonDict = {"parent": parent, "textures": {"layer0": f"{self.namespace}:item/{variant}"}}
+								pull_content: JsonDict = {"parent": parent, "textures": {"layer0": f"{self.ns}:item/{variant}"}}
+
 								# Add texture to assets
-								if variant + ".png" in self.source_textures:
-									Mem.ctx.assets[f"{self.namespace}:item/{variant}"] = Texture(source_path=self.source_textures[variant + ".png"])
+								variant_png: str = variant + ".png"
+								if variant_png in self.source_textures:
+									Mem.ctx.assets[self.ns].textures[f"item/{variant}"] = texture_mcmeta(self.source_textures[variant_png])
+
 								# Add model to assets
-								Mem.ctx.assets[f"{self.namespace}:item/{self.item_name}_pulling_{i}"] = Model(super_json_dump(pull_content, max_level=4))
+								Mem.ctx.assets[self.ns].models[f"item/{self.item_name}_pulling_{i}"] = set_json_encoder(Model(pull_content), max_level=4)
 
 								if i < (len(sorted_pull_variants) - 1):
 									pull: float = 0.65 + (0.25 * i)
-									model: str = f"{self.namespace}:item/{self.item_name}_pulling_{i + 1}"
+									model: str = f"{self.ns}:item/{self.item_name}_pulling_{i + 1}"
 									items_content["model"]["on_true"]["entries"].append({ # type: ignore
 										"model": {
 											"type": "minecraft:model",
@@ -317,7 +404,7 @@ class AutoModel:
 									})
 
 							# Add the items/bow.json file
-							Mem.ctx.assets[f"{self.namespace}:{self.item_name}{on_off}"] = set_json_encoder(ItemModel(items_content), max_level=4)
+							Mem.ctx.assets[self.ns].item_models[self.item_name + on_off] = set_json_encoder(ItemModel(items_content), max_level=4)
 
 			# Add overrides
 			for key, value in overrides.items():
@@ -333,12 +420,10 @@ class AutoModel:
 					if (texture.split("/")[-1] + on_off) in variants:
 						content["textures"][key] = texture + on_off
 
-			# Add used textures (ignore minecraft namespace)
-			if content.get("textures"):
-				for texture in content["textures"].values():
-					if texture.startswith("minecraft:"):
-						continue
-					self.used_textures.add(texture)
+			# Remove empty textures
+			if exclude_textures or not content.get("textures"):
+				if "textures" in content:
+					del content["textures"]
 
 			# Copy used textures
 			if not exclude_textures and content.get("textures"):
@@ -350,26 +435,30 @@ class AutoModel:
 					texture_name = texture.split(":")[-1].split("/")[-1]  # Get just the filename
 					texture_name += ".png"
 					if texture_name in self.source_textures:
-						mcmeta: JsonDict | None = None
-						if os.path.exists(self.source_textures[texture_name] + ".mcmeta"):
-							mcmeta = super_json_load(self.source_textures[texture_name] + ".mcmeta")
-						Mem.ctx.assets[texture] = Texture(source_path=self.source_textures[texture_name], mcmeta=mcmeta)
+						Mem.ctx.assets[texture] = texture_mcmeta(self.source_textures[texture_name])
 					else:
 						if not self.ignore_textures:
 							raise ValueError(f"Texture '{texture_name}' not found in source textures")
 
-			# Remove empty textures
-			if exclude_textures or not content.get("textures"):
-				if "textures" in content:
-					del content["textures"]
+			# Add used textures
+			if content.get("textures"):
+				for key, texture in content["textures"].items():
+					if texture.startswith("minecraft:"):
+						self.used_minecraft_textures.add(texture)
+						content["textures"][key] = to_atlas(texture)
+					else:
+						self.used_textures.add(texture)
 
 			# Add model to assets
 			if self.data.get(OVERRIDE_MODEL, None) != {}:
-				Mem.ctx.assets[f"{self.namespace}:item/{self.item_name}{on_off}"] = set_json_encoder(Model(content), max_level=4)
+				Mem.ctx.assets[self.ns].models[f"item/{self.item_name}{on_off}"] = set_json_encoder(Model(content), max_level=4)
 			Mem.ctx.meta["stewbeet"]["rendered_item_models"].add(self.data["item_model"])
 
 			# Generate the json file required in items/
 			if not self.data["id"].endswith("bow"):
-				items_model = {"model": {"type": "minecraft:model", "model": f"{self.namespace}:item/{self.item_name}{on_off}"}}
-				Mem.ctx.assets[f"{self.namespace}:{self.item_name}{on_off}"] = set_json_encoder(ItemModel(items_model), max_level=4)
+				items_model = {"model": {"type": "minecraft:model", "model": f"{self.ns}:item/{self.item_name}{on_off}"}}
+				Mem.ctx.assets[self.ns].item_models[self.item_name + on_off] = set_json_encoder(ItemModel(items_model), max_level=4)
+
+		# Return
+		return self.used_minecraft_textures
 
