@@ -8,14 +8,16 @@ from beet import Context
 from beet.core.utils import JsonDict
 from stouputils.decorators import measure_time
 from stouputils.io import relative_path, super_json_dump, super_open
-from stouputils.print import debug, error, info, progress
+from stouputils.print import debug, error, info, progress, warning
 
 from ...core.__memory__ import Mem
 from ...core.constants import (
 	CATEGORY,
 	CUSTOM_BLOCK_ALTERNATIVE,
 	CUSTOM_BLOCK_VANILLA,
+	GROWING_SEED,
 	NO_SILK_TOUCH_DROP,
+	PAINTING_DATA,
 	RESULT_OF_CRAFTING,
 	USED_FOR_CRAFTING,
 	VANILLA_BLOCK,
@@ -62,8 +64,18 @@ def beet_default(ctx: Context) -> None:
 
 	# Check every single thing in the definitions
 	errors: list[str] = []
+	warnings: list[str] = []
 	for item, data in Mem.definitions.items():
 
+		# Check if components doesn't start with "minecraft:", if so rename them
+		to_replace: bool = False
+		for k in data.keys():
+			if k.startswith("minecraft:"):
+				warnings.append(f"Remove the 'minecraft:' prefix from key '{k}' in item '{item}', as it's automatically added by stewbeet.")
+				to_replace = True
+		if to_replace:
+			Mem.definitions[item] = data = {k.replace("minecraft:", "", 1) if k.startswith("minecraft:") else k: v for k, v in data.items()}
+
 		# Check if the item uses a reserved name
 		if item == "heavy_workbench":
 			errors.append(f"'{item}' is reserved for the heavy workbench used for NBT recipes, please use another name")
@@ -107,8 +119,6 @@ def beet_default(ctx: Context) -> None:
 		# Check for wrong custom ores data
 		if data.get(VANILLA_BLOCK) == VANILLA_BLOCK_FOR_ORES and not data.get(NO_SILK_TOUCH_DROP):
 			errors.append(f"NO_SILK_TOUCH_DROP key missing for '{item}', should be the ID of the block that drops when mined without silk touch")
-		if data.get(VANILLA_BLOCK) != VANILLA_BLOCK_FOR_ORES and data.get(NO_SILK_TOUCH_DROP):
-			errors.append(f"NO_SILK_TOUCH_DROP key should not be used for '{item}' if it doesn't use VANILLA_BLOCK_FOR_ORES")
 		if data.get(NO_SILK_TOUCH_DROP):
 			no_silk_drop = data[NO_SILK_TOUCH_DROP]
 			if isinstance(no_silk_drop, str):
@@ -151,15 +161,6 @@ def beet_default(ctx: Context) -> None:
 				for i, line in enumerate(lore):
 					if not isinstance(line, dict | list | str):
 						errors.append(f"Line #{i} in 'lore' key should be a Text Component for '{item}', ex: {{\"text\":\"This is a lore line\"}} or [\"This is a lore line\"]")
-					else:
-						# Verify format {"text":"..."} or "..."
-						line = str(line)
-						if not (line.startswith('{') and line.endswith('}')) \
-							and not (line.startswith('[') and line.endswith(']')) \
-							and not (line.startswith('"') and line.endswith('"')) \
-							and not (line.startswith("'") and line.endswith("'")) \
-							and not line == "":
-							errors.append(f"Item '{item}' has a lore line that is not in a correct text component format: {line}\n We recommend using 'https://misode.github.io/text-component/' to generate the text component")
 
 		# Check all the recipes
 		if data.get(RESULT_OF_CRAFTING) or data.get(USED_FOR_CRAFTING):
@@ -237,13 +238,88 @@ def beet_default(ctx: Context) -> None:
 							if not recipe.get("cookingtime") or not isinstance(recipe["cookingtime"], int):
 								errors.append(f"Recipe #{i} in RESULT_OF_CRAFTING should have an int 'cookingtime' key for '{item}'")
 
-					# Check the result count
-					if (not recipe.get("result_count") or not isinstance(recipe["result_count"], int)) and (recipe.get("type") not in ("smithing_trim",)):
-						errors.append(f"Recipe #{i} in RESULT_OF_CRAFTING should have an int 'result_count' key for '{item}'")
+		# Check the PAINTING_DATA key
+		if data.get(PAINTING_DATA):
+			if data["id"] != "minecraft:painting":
+				errors.append(f"PAINTING_DATA key should only be used with 'id' equal to 'minecraft:painting' for '{item}', found '{data['id']}' instead")
+			elif not isinstance(data[PAINTING_DATA], dict):
+				errors.append(f"PAINTING_DATA key should be a dictionary for '{item}'")
+			else:
+				painting_data = cast(JsonDict, data[PAINTING_DATA])
+				if painting_data.get("author") and not isinstance(painting_data["author"], dict | list | str):
+					errors.append(f"PAINTING_DATA 'author' key should be a Text Component for '{item}', got '{painting_data['author']}'")
+				if painting_data.get("title") and not isinstance(painting_data["title"], dict | list | str):
+					errors.append(f"PAINTING_DATA 'title' key should be a Text Component for '{item}', got '{painting_data['title']}'")
+				if not painting_data.get("width") or not isinstance(painting_data["width"], int) or painting_data["width"] < 1 or painting_data["width"] > 16:
+					errors.append(f"PAINTING_DATA should have an integer 'width' key between 1 and 16 for '{item}'")
+				if not painting_data.get("height") or not isinstance(painting_data["height"], int) or painting_data["height"] < 1 or painting_data["height"] > 16:
+					errors.append(f"PAINTING_DATA should have an integer 'height' key between 1 and 16 for '{item}'")
+
+				# Add missing data if needed
+				if "painting/variant" not in data:
+					data["painting/variant"] = f"{ctx.project_id}:{item}"
+
+		# Check the GROWING_SEED key
+		if data.get(GROWING_SEED):
+			if not isinstance(data[GROWING_SEED], dict):
+				errors.append(f"GROWING_SEED key should be a dictionary for '{item}'")
+			else:
+				growing_seed: JsonDict = data[GROWING_SEED]
+				if not growing_seed.get("seconds") or not isinstance(growing_seed["seconds"], int):
+					errors.append(f"GROWING_SEED should have an integer 'seconds' key for '{item}'")
+				if not growing_seed.get("loots") or not isinstance(growing_seed["loots"], list):
+					errors.append(f"GROWING_SEED should have a list 'loots' key for '{item}'")
+				else:
+					loots = cast(list[Any], growing_seed["loots"])
+					for i, loot in enumerate(loots):
+						if not isinstance(loot, dict):
+							errors.append(f"GROWING_SEED loot #{i} should be a dictionary for '{item}'")
+						else:
+							loot = cast(JsonDict, loot)
+							if not loot.get("id") or not isinstance(loot["id"], str):
+								errors.append(f"GROWING_SEED loot #{i} should have a string 'id' key for '{item}'")
+							if loot.get("rolls"):
+								rolls = loot["rolls"]
+								if isinstance(rolls, int):
+									pass # Integer is valid
+								elif isinstance(rolls, dict):
+									rolls_dict = cast(JsonDict, rolls)
+									if not rolls_dict.get("type") or not isinstance(rolls_dict["type"], str):
+										errors.append(f"GROWING_SEED loot #{i} 'rolls' dict should have a string 'type' key for '{item}', ex: {{\"type\":\"minecraft:uniform\",\"min\":1,\"max\":2}}")
+								else:
+									errors.append(f"GROWING_SEED loot #{i} 'rolls' should be an integer or dict for '{item}', ex: 1 or {{\"type\":\"minecraft:uniform\",\"min\":1,\"max\":2}}")
+							else:
+								loot["rolls"] = 1 # Default rolls to 1 if not present
+
+							# Check fortune key (optional)
+							if loot.get("fortune"):
+								fortune = loot["fortune"]
+								if not isinstance(fortune, dict):
+									errors.append(f"GROWING_SEED loot #{i} 'fortune' should be a dictionary for '{item}'")
+								else:
+									fortune_dict = cast(JsonDict, fortune)
+									if (not fortune_dict.get("extra") and fortune_dict.get("extra") != 0) or not isinstance(fortune_dict.get("extra"), int | float):
+										errors.append(f"GROWING_SEED loot #{i} 'fortune' dict should have a numeric 'extra' key for '{item}', ex: {{\"extra\":0,\"probability\":0.1}}")
+									if (not fortune_dict.get("probability") and fortune_dict.get("probability") != 0) or not isinstance(fortune_dict.get("probability"), float | int):
+										errors.append(f"GROWING_SEED loot #{i} 'fortune' dict should have a numeric 'probability' key for '{item}', ex: {{\"extra\":0,\"probability\":0.1}}")
+				if growing_seed.get("texture_basename") and not isinstance(growing_seed["texture_basename"], str):
+					errors.append(f"GROWING_SEED 'texture_basename' key should be a string for '{item}'")
+				elif not growing_seed.get("texture_basename"):
+					growing_seed["texture_basename"] = item
+				if growing_seed.get("planted_on"):
+					if not isinstance(growing_seed["planted_on"], str):
+						errors.append(f"GROWING_SEED 'planted_on' key should be a string for '{item}'")
+					elif "minecraft:" in growing_seed["planted_on"]:
+						warnings.append(f"Remove the 'minecraft:' prefix from 'planted_on' in GROWING_SEED for '{item}', as it's automatically added by stewbeet.")
+						data[GROWING_SEED]["planted_on"] = growing_seed["planted_on"].replace("minecraft:", "")
+				else:
+					growing_seed["planted_on"] = "dirt" # Default to dirt if not present
 
 	# Log errors if any
 	if errors:
 		error("Errors found in the definitions during verification:\n- " + "\n- ".join(errors))
+	elif warnings:
+		warning("Warnings found in the definitions during verification:\n- " + "\n- ".join(warnings))
 	else:
 		info("No errors found in the definitions during verification")
 
